import numpy as np

from topos.core.mesh import Mesh
from .widgets import ModelWidget


class PMesh(Mesh):
    """A Preview Mesh object

    This extends the core Mesh object by managing an instance of the
    ModelWidget to allow for in notebook previews of the Mesh's geometry
    """

    def __init__(self, verts=None, faces=None, name=None):
        super().__init__(verts, faces, name)

        self.widget = ModelWidget()
        self.widget.vertices = self.wverts
        self.widget.faces = self.wfaces

    @classmethod
    def frommesh(cls, mesh):
        """Create a PMesh from an existing mesh."""

        verts = mesh.vertices
        faces = mesh.faces
        name = mesh.name

        return cls(verts, faces, name)

    @property
    def wverts(self):
        """Return vertices formatted in a way compatible with the widget."""

        if self._verts is not None:
            verts = self.vertices.cartesian
            return [list(v) for v in verts]

        return []

    @property
    def wfaces(self):
        """Return faces formatted in a way compatible with the widget."""

        # Annoyingly three.js no longer support quads...
        # so we have to triangulate each face...
        # We represent each face as follows
        #
        # 4 -------- 3
        # |          |
        # |          |
        # |          |
        # |          |   face = [ 1, 2, 3, 4 ]
        # 1 -------- 2
        #
        # So for each face we will have to emit 2 triangles
        #
        # 4 -------- 3
        # | ⋱        |     ⋱ = U+22F1
        # |   ⋱      |
        # |     ⋱    |      lower = [1, 2, 4]
        # |       ⋱  |      upper = [4, 2, 3]
        # 1 -------- 2
        faces = []

        if self._faces is not None:
            fs = self.faces.data

            # We have to renumber the faces, this is due to threejs
            # starts numbering vertices at 0, whereas blender starts
            # at 1 :/
            #
            # We create a new array so that we don't mess with the one
            # we use internally
            FS = np.array(fs)
            FS = FS - 1

            for f in FS:

                # Lower triangle
                faces.append([f[0], f[1], f[3]])

                # Upper triangle
                faces.append([f[3], f[1], f[2]])

        return faces

    def show(self):
        return self.widget


import ipywidgets as widgets
from traitlets import Unicode, List

from .version import __version__


@widgets.register
class HelloWidget(widgets.DOMWidget):

    _view_name = Unicode("HelloView").tag(sync=True)
    _view_module = Unicode("topos-previewjs").tag(sync=True)
    _view_module_version = Unicode(__version__).tag(sync=True)

    value = Unicode("Hello, World!").tag(sync=True)


@widgets.register
class ModelWidget(widgets.DOMWidget):

    _view_name = Unicode("ModelView").tag(sync=True)
    _view_module = Unicode("topos-previewjs").tag(sync=True)
    _view_module_version = Unicode(__version__).tag(sync=True)

    vertices = List([[1., 2., 3.], [4., 5., 6.]]).tag(sync=True)
    faces = List([[1, 2, 3, 4], [5, 6, 7, 8]]).tag(sync=True)



from .pmesh import PMesh
from .version import __version__


def _jupyter_nbextension_paths():
    return [{
        'section': 'notebook',
        'src': 'static',
        'dest': 'topos-previewjs',
        'require': 'topos-previewjs/extension'
    }]



from setuptools import setup, find_packages
from setuptools.command.install import install
from setuptools.command.develop import develop
from subprocess import check_call


class PostInstall(install):
    """Post installation steps.

    1. Install the notebook extension using jupyter nbextension install
    2. Enable the extension using jupyter nbextension enable
    """

    def run(self):
        # Do the usual install step
        install.run(self)

        # Register and install ourselves with jupyter
        check_call(["jupyter", "nbextension", "install",
                    "--py", "--symlink", "--sys-prefix", "topos.ext.preview"])
        check_call(["jupyter", "nbextension", "enable",
                    "--py", "--sys-prefix", "topos.ext.preview"])


def readme():
    with open("README.rst") as f:
        return f.read()


exec(open('topos/ext/preview/version.py', 'r').read())


setup(name='topos-preview',
      version=__version__,
      description="Widget for Jupyter Notebooks that allows for previewing topos Mesh objects",
      long_description=readme(),
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3 :: Only',
          'Topic :: Multimedia :: Graphics'
      ],
      author='Alex Carney',
      author_email="alcarneyme@gmail.com",
      license='MIT',
      packages=['topos.ext.preview'],
      install_requires=[
          'numpy',
          'topos',
          'jupyter'
      ],
      setup_requires=['pytest-runner'],
      test_suite='tests',
      tests_require=['pytest', 'hypothesis'],
      cmdclass={
          'install': PostInstall
      },
      python_requires='>=3.0',
      include_package_data=True,
      zip_safe=False)
